(ns kube-deploy.blocks
  (:require [clojure.java.io :as io]
            [kube-deploy.helpers :as lkh]

            [lambdakube.core :as lk]))

(def resources-mini
  {:requests
   {:cpu "100m" :memory "64Mi"}
   :limits
   {:cpu "200m" :memory "256Mi"}})

(def resources-small
  {:requests
   {:cpu "100m" :memory "100Mi"}
   :limits
   {:cpu "200m" :memory "400Mi"}})

(def resources-medium
  {:requests
   {:cpu "100m" :memory "500Mi"}
   :limits
   {:cpu "200m" :memory "1Gi"}})


(defn postgres 
  ([kube-injector]
   (postgres kube-injector {}))
  ([kube-injector settings]
   (let [app-name (get settings :app-name "postgres")
         app-pod-name (str app-name "-pod")
         app-rule-name (str app-name "-rule")
         app-container-name (str app-name "-container")
         app-dependencies (get  settings :dependencies [])
         app-containers 1
         app-labels (get  settings :labels {:app "postgres"})
         app-image (get settings :image "postgres")
         app-resources (get  settings :resources resources-small)]
  (-> kube-injector
      (lk/rule :postgres-master-rule []
               (fn []
                 (-> (lk/pod
                      app-pod-name
                      app-labels
                      )
                     (lk/add-container app-container-name app-image
                                       (-> {}
                                           (lkh/add-to-spec :resources app-resources)
                                           (lkh/add-readiness-probe-cmd ["pg_isready"])
                                           (lkh/add-liveness-probe-cmd ["pg_isready"])
                                           (lkh/add-to-spec :envFrom [{:secretRef {:name "environment"}}])))
                     (lk/stateful-set 1)

                     (lk/add-volume-claim-template :postgres-volume
                                                   {:accessModes ["ReadWriteOnce"]
                                                    :resources {:requests {:storage "1Gi"}}}
                                                   {:bar "/var/lib/postgresql/data"})
                     (lk/expose-cluster-ip app-pod-name
                                           (lk/port :postgres :postgres 5432 5432)))))))))


(defn django-gunicorn
  ([kube-injector]
   (django-gunicorn kube-injector {}))
  ([kube-injector settings]
   (let [app-name (get settings :app-name "django-gunicorn")
         app-pod-name (str app-name "-pod")
         app-rule-name (str app-name "-rule")
         app-gunicorn-container-name (str app-name "-container")
         app-nginx-container-name (str app-name "-nginx-container")
         app-dependencies (get settings :dependencies [])
         app-containers 1
         app-labels (get  settings :labels {:app "django-gunicorn"})
         app-image (get settings :image "python:3")
         app-resources (get  settings :resources resources-small)]

     (-> kube-injector
         (lk/rule (keyword (str app-name "-rule")) app-dependencies
                  (fn []
                    (-> (lk/pod
                         app-pod-name
                         app-labels
                         {:imagePullSecrets [{:name "docker-registry"}]})
                                        ; Add containers to the pod, one for frontend assets and one for backend
                        (lk/add-container
                         app-gunicorn-container-name
                         app-image
                         {:resources app-resources
                          :imagePullPolicy "Always"
                          :command (get settings :command ["/usr/bin/gunicorn"
                                                           "wsgi"
                                                           "--workers" "2"
                                                           "-k" "gevent"
                                                           "--worker-connections" "100"
                                                           "--max-requests" "300"
                                                           "--keep-alive" "1000"
                                                           "-b" "0.0.0.0:5000"
                                                           "--chdir=/app"])
                          :ports [{:containerPort 5000 :name "gunicorn-port"}]
                          :envFrom [{:secretRef {:name "environment"}}]
                          #_(lkh/add-readiness-probe-cmd ["wget" "127.0.0.1:5000"])
                          #_(lkh/add-liveness-probe-cmd ["wget" "127.0.0.1:5000"])})

                        (lk/add-volume "static-files" {:emptyDir {}}
                                       {app-pod-name "/app/staticfiles"})

                                        ; order does matter here, deployment after initcontainers
                                        ;(lku/wait-for-service-port postgres :postgres)
                                        ;(lku/wait-for-service-port redis :redis)

                                        ; wrap in a deployment
                        (lk/deployment app-containers)
                        (lk/expose-cluster-ip app-gunicorn-container-name
                                              (lk/port app-pod-name :gunicorn-port 5000 5000))
                        (lk/expose-cluster-ip app-nginx-container-name
                                              (lk/port app-pod-name :nginx-port 80 80)))))

         ; inject the ingress required to serve to gunicorn app
         (lk/rule (keyword (str app-name "-ingress-rule")) []
                  (fn []
                    (let [ingress-name (str app-name "-ingress")]
                      (-> (lkh/ingress ingress-name {:app :nginx-ingress :env :production :tier :ingress})
                          (lkh/add-spec :tls [{:hosts [(:ingress-host settings)]
                                               :secretName "letsencrypt-production"}])
                          (lkh/add-spec :rules [{:host (:ingress-host settings)
                                                 :http {:paths
                                                        [{:path "/"
                                                          :backend
                                                          {:serviceName "gunicorn-port"
                                                           :servicePort 5000}}
                                                         {:path "/static/"
                                                          :backend
                                                          {:serviceName "nginx-port"
                                                           :servicePort 80}}]}}])
                          (lk/add-annotation :certmanager.k8s.io/cluster-issuer "letsencrypt-production")
                          (lk/add-annotation :kubernetes.io/ingress.class "nginx")))))))))
